@@ -1,7 +1,7 @@
 library("testthat")
 library("modelStudio")
 library("DALEX")
-library("randomForest")
+library("ranger")
 library("xgboost")
 
 test_check("modelStudio")
